<?php

/**
 * Krumo - replacement for print_r() and var_dump().
 * https://github.com/mmucklo/krumo
 * https://github.com/oodle/krumo
 *
 * This is not required for the ApiConnection to work
 * it is provided to visualise the API results better.
 */
require_once('krumo/class.krumo.php');

/**
 * API Examples.
 *
 * There are various dump's through this file so you can see the 
 * results of each API call. These are generated by krumo for ease.
 */

/**
 * Create new instance of ApiConnection.
 *
 * This needs to be done in each section you are firing requests from.
 * First we include the api-connection.php class file, then we create
 * a new instance of the ApiConnection class.
 */
require_once('inc/api-connection.php');

$api = new ApiConnection(
	'API_ID.', // The API ID
	'API_KEY', // The API Key
	'https://control.example.com/rest' // The API Base URL - Note: theres no trailing slash!
);


/**
 * Getting users.
 */
$results = $api->get('/users/details', array(
	'email_address' => 'terry@email.com'
));
krumo($results);


/**
 * Updating users.
 * In this example we'll update the first and last name.
 */
$results = $api->post('/users/add_or_update', array(
	'email_address' => 'terry@email.com',
	'first_name' => 'Terry',
	'last_name' => 'Tibbs'
));
krumo($results);


/**
 * Importing lists.
 *
 * Importing lists is super easy. In the following example first we 
 * check for field maps we have avaliable. For the list we're importing
 * we want the default 'Email addresses only' map.
 * Then we proceed create to create the list. If we get a 200/valid = true back
 * then we import the list from the example list CSV using the CURLFile class.
 *
 * Further Reading:
 * http://php.net/manual/en/class.curlfile.php
 * Check the Field Maps > Lists, Users & Targeting documentation
 */
$results = $api->get('/field_maps/list');
krumo($results);
/**
 * array (size=2)
 * 		'field_map_api_identifier' => string 'SYS-1' (length=5)
 *   	'field_map_name' => string 'Email address only' (length=18)
 *
 * Thats the one we want. Note: field_map_api_identifier.
 */
// Create the list
$results = $api->post('/lists/add_or_update', array(
	'list_api_identifier' => 'api_examples',
	'list_description' => 'Terrys List'
));
krumo($results);

// If the list was created successfully...
if ($results['valid'] === true) {

	// Create a cURL file
	$list = new CurlFile('assets/test-list.csv', 'text/csv');

	// Post the list
	$results = $api->post('/lists/import_optin', array(
		'list_api_identifier' => 'api_examples',
		'field_map_api_identifier' => 'SYS-1',
		'import_resubscribe' => 'Y',
		'import_file' => $list
	));
}
krumo($results);


/**
 * Campaign Reporting.
 *
 * In this example we'll see which campaigns have run over
 * the past 30 days. From the data we get back we'll grab some
 * reports related to that campaign and finally download the data
 * for the last campaign.
 *
 * Further Reading:
 * http://php.net/manual/en/function.header.php
 */
$results = $api->get('/campaigns/search', array(
	'date_from' => date('Y-m-d', strtotime("-30 days")),
	'date_to' => date('Y-m-d'),
));
krumo($results);


// If we are successful...
if ($results['valid'] === true) {

	// Get the last campaign
	$last_campaign = $results['response'][0];

	// Now we can get some stats
	$results = $api->get('/campaigns/details', array(
		'campaign_id' => $last_campaign['campaign_id']
	));

	// Now we have more details info such as the stats of the campaign
	krumo($results);

	// Now we'll download some campaign data
	$results = $api->get('/campaigns/download', array(
		'campaign_id' => $last_campaign['campaign_id'],
		'export_type' => 'sent',
		'filename' => 'last_campaign_data',
	));

	if ($results['valid'] === true) {

		krumo($results);
		// Set the headers - This is commented for ease
		// Once the comments are removed this will download a report of the last campaign.
		// 
		// header('Content-Description: File Transfer');
		// header('Content-Type: ' . $results['response']['content_type']);
		// header('Content-Disposition: attachment; filename="' . $results['response']['content_name'] . '"');
		// header('Expires: 0');
		// header('Cache-Control: must-revalidate');
		// header('Pragma: public');
		// header('Content-Length: ' . $results['response']['content_length']);

		// // Echo the content
		// echo $results['response']['content'];
	}
}

/**
 * Triggering Workflows.
 *
 * Triggering workflows is incredibly easy using the API. We
 * can also pass through a JSON encoded array of vars which
 * we can use within Workflows.
 *
 * Trigger requests will be rejected unless the Workflow is
 * 'active' or is 'paused' and queueing is permitted. Valid
 * requests return an internal Workflow ID which can then be
 * used to track and report on.
 */
$session_vars = json_encode(array('data' => 'Heres some custom session vars!'));

$results = $api->post('/automation/trigger_workflow', array(
	'workflow_api_identifier' => 'the_workflow_name',
	'workflow_session_variables' => $session_vars,
	'email_address' => 'terry@email.com'
));
krumo($results);

/**
 * Template Upload.
 *
 * Here we will upload a template using the API. This uses one of
 * the boilerplate templates and include HTML and Images.
 *
 * We need to create a few cURL files before we start, then post
 * those to the API.
 *
 * Further Reading:
 * http://php.net/manual/en/function.header.php
 * http://php.net/manual/en/class.curlfile.php
 */

// Create the cURL files...
$html = new CurlFile('assets/template/index.html', 'text/html');
$img = new CurlFile('assets/template/img.zip', 'application/zip');

// Make the request
$results = $api->post('/templates/upload', array(
	'template_api_identifier' => 'test_template_upload',
	'template_description' => 'API Uploaded Template',
	'template_subject' => 'API Template',
	'email_profile_api_identifier' => 'test_email_profile',
	'primary_content_type' => 'HTML',
	'primary_file' => $html,
	'image_archive' => $img,
));
krumo($results);